/*

-Procedure rotfrm
 
     Obtain a matrix to transform a vector in a SPICE reference
     frame to a vector referenced in a new frame having the following
     characteristics:

     1) The Z-axis of the new frame is parallel to central body CENID's
        spin axis (TYPE='equ'), or to the angular momentum
        vector of CENID's orbital motion (TYPE='orb').

     2) The reference body REFID lies in the plane formed by the
        X-axis and Z-axis of the new frame.

     3) Note: If INFRAM is a non-inertial reference frame, results
        may be meaningless.   Use J2000, or any other of the SPICE
        inertial frames such as B1950, ECLIPJ2000, etc., for
        best results.


     Jeff Bytof, last updated 11-May-2001

Usage:
------

  ( In this example the input vector is referenced to J2000 and the
    output vector is referenced to the Titan equatorial frame, 
    with the Sun as the secondary reference aligned with the new
    frame's X-axis. )

              #define infram "J2000"
              #define type   "equ" 
              
              cenid = 606; 
              refid =  10;
 
              et = 34533232.0;
              
              rotfrm ( cenid,  refid,  infram, et, type,
                       mat,   &found, &ier                ); 

              mxv_c ( mat, vec_2000, vec_equ ); 


*/ 
            


#include "SpiceUsr.h"
#include "SpiceZmc.h"
#include "SpiceZfc.h"

#define ROOM 50

void rotfrm ( SpiceInt          cenid,
              SpiceInt          refid, 
              ConstSpiceChar  * infram,
              SpiceDouble       et, 
              ConstSpiceChar  * type, 
              SpiceDouble       mat [3][3], 
              SpiceBoolean    * found, 
              SpiceInt        * ier     )

{

/*

-Brief_I/O

    Variable        I/O         Description 
    --------        ---         -----------
    cenid            I          NAIF ID code of central body.
    refid            I          NAIF ID code of reference body.  This
                                   body determines the direction of the 
                                   new frame's x-axis. 
    infram           I          Name of SPICE-defined initial reference frame.
    et               I          Ephemeris time epoch of transformation. 
    type             I          Type of reference frame - 
                                  'orb' for planet orbital, 
                                  'equ' for planet equatorial.
    mat              O          Rotation matrix which transforms a vector
                                  expressed in the INFRAM reference frame
                                  to one in the body-fixed equatorial or
                                  orbital frame.
    found            O          Returns SPICETRUE if a valid matrix is returned.    ier              O          Returns zero if a matrix is found, otherwise
                                   returns a positive integer error code.

*/

/*
   Local variables.
*/

   SpiceChar     cenfrm   [20];

   SpiceDouble   bodfxd   [3][3];
   SpiceDouble   frmref   [3];
   SpiceDouble   lt; 
   SpiceDouble   minvel   [3];
   SpiceDouble   state    [6];
   SpiceDouble   vel      [3];
   SpiceDouble   toref    [3];
   SpiceDouble   x        [3];
   SpiceDouble   y        [3];
   SpiceDouble   zhat     [] = { 0.0, 0.0, 1.0 };
   SpiceDouble   znorth   [3];
   SpiceDouble   zorb     [3];

   SpiceInt      frcode;
   SpiceInt      i;
   SpiceInt      j;
   
   SpiceBoolean  fnd;

   *found = SPICEFALSE;
   *ier   = 0;


/*
   Check for obvious errors on input.
*/

   if ( cenid == refid ) { 
/*
   Central body and reference body IDs are the same.
*/ 
      *ier = 1;
      return;
   }


   if ( ( strcmp( type, "orb") != 0 ) && 
        ( strcmp( type, "equ") != 0 ) ) {
/*
   Return from bad output frame type.
*/
      *ier = 2;
      return;
   } 


/*
   Obtain unit vector to reference body from central body.
*/

   spkez_c  ( refid,   et, infram, "LT+S", cenid, state, &lt );
   vpack_c  ( state[0], state[1], state[2], toref );
   vhat_c   ( toref, toref );


/**********************************************
   Construct orbital reference frame.
*/

   if ( strcmp( type, "orb" ) == 0 ) {

/*
   Get the vector normal to the orbit.

   Sign of state vector has to be reversed to get 
   the correct angular momentum.
*/

      vpack_c  ( state[3], state[4], state[5], vel );
      vminus_c ( vel,      minvel                  );
      vminus_c ( toref,    frmref                  );
      ucrss_c  ( frmref,   minvel,   zorb          ); 

/*
   Construct the new frame's basis vectors.
*/

     ucrss_c ( zorb, toref, y );

/*
   Assemble the rotation matrix.
*/

     for ( i=0; i<3; i++ ) {
        mat[0][i] = toref[i];
        mat[1][i] = y[i];
        mat[2][i] = zorb[i];
     }
  
 
/**********************************************
   Construct equatorial reference frame.
*/

   } else if (  strcmp( type, "equ" ) == 0 ) {

/*
   Get the north pole vector of the central body.
*/

      cidfrm_c ( cenid, ROOM, &frcode, cenfrm, &fnd ); 

      if ( fnd != SPICETRUE ) { 
         
/*
   Return if frame for central body not found.
*/
         *ier = 3;
         return;
      }

      pxform_c ( cenfrm, infram, et, bodfxd );

      mxv_c ( bodfxd, zhat, znorth );

/*
   Construct the basis vector set.
*/

      ucrss_c ( znorth, toref, y );
      ucrss_c ( y, znorth, x );

/*
   Assemble the rotation matrix.
*/

      for ( i=0; i<3; i++ ) {
         mat[0][i] = x[i];
         mat[1][i] = y[i];
         mat[2][i] = znorth[i];
      }

   }  

      

   *found = SPICETRUE; 

   return;
}
